package com.ibm.tools.attach.javaSE;
/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2009, 2010  All Rights Reserved
 */

import java.io.File;
import java.io.IOException;

class TargetDirectory {

	/*
	 * allow owner or group to create, read directories, but only owner can delete.
	 * Need to allow other processes to lock the sync file.
	 */
	static final int ADVERTISEMENT_FILE_PERMISSIONS = 0600;
	static final String ATTACH_NOTIFICATION_SYNC_FILENAME = "attachNotificationSync"; //$NON-NLS-1$
	static final String PID_PATTERN = "\\A\\d+\\z"; //$NON-NLS-1$
	static final int SYNC_FILE_PERMISSIONS = 0777; /* All users must have write access in order to get an exclusive (i.e write) lock on the file */
	static final int TARGET_DIRECTORY_PERMISSIONS = 01711;
	/**
	 * Create the directory and files specific to this VM
	 * @param myVmId proposed ID of this VM
	 * @return actual VM ID
	 * @throws IOException if files or directory cannot be created
	 */
	static String createTargetIPCDir(String myVmId, boolean recycle) throws IOException {
		String newId = myVmId;
		File tgtDir = new File(getTargetDirectoryPath(newId));
		int count = 0;

		if (!myVmId.matches(PID_PATTERN) && !recycle) {
			while (tgtDir.exists() && (count < 100)) {
				File oldAdvert = new File(tgtDir, Advertisement.getFilename());
				if (!oldAdvert.exists()) {
					/* this is a stale directory with no advertisement */
					break;
				}
				newId = myVmId+'_'+count;
				tgtDir = new File(getTargetDirectoryPath(newId));
				++count;
			}
		}
		if (!tgtDir.exists()) {
			IPC.mkdirWithPermissions(tgtDir.getAbsolutePath(), TARGET_DIRECTORY_PERMISSIONS);
		} else {
			// K0547 = Attach API target directory already exists for VMID {0}
			IOException e = new IOException(com.ibm.oti.util.Msg.getString("K0547", myVmId));  //$NON-NLS-1$
			AttachHandler.setFactoryException(e);
			throw e;
		}
		TargetDirectory.createNotificationSyncFile(newId);
		return newId;
	}

	/**
	 * @param myVmId machine-friendly name of the VM
	 * @return directory of the target's notification directory, or null in case of error
	 */
	static String getTargetDirectoryPath(String myVmId) {
		File cd = CommonDirectory.getCommonDirFileObject();
		if ((null == myVmId) || (null == cd)) {
			/* either commonDir or the VMID have not been properly initialized */
			return null;
		}
		return (new File(cd, myVmId)).getPath();
	}

	/**
	 * create a lock file for the target
	 * @param myVmId id for the target VM
	 * @throws IOException if the file cannot be created
	 */
	static void createNotificationSyncFile(String myVmId) throws IOException {
		File notificationSyncFile = TargetDirectory.getNotificationSyncFile(myVmId);
		if (!notificationSyncFile.exists()) {
			if (AttachHandler.isAttachApiTerminated()) {
				/* don't create the file if we don't need it */
				return;
			}
			if (!notificationSyncFile.createNewFile()) {
				IPC.logMessage(notificationSyncFile.getName(), " already exists"); //$NON-NLS-1$
			}
			/* always do the chmod in case the file got changed accidentally */
			IPC.chmod(notificationSyncFile.getAbsolutePath(), TargetDirectory.SYNC_FILE_PERMISSIONS);
			/* AttachHandler.terminate() will delete this file on shutdown */
		}
	}

	/**
	 * convenience method for getNotificationSyncFile, using the local value of the VM ID
	 * @return File object to the target's sync file or null if the common directory is missing
	 */
	static File getNotificationSyncFile() {
		String tdp = TargetDirectory.getTargetDirectoryPath(AttachHandler.getVmId());
		if (null == tdp) {
			return null;
		}
		String inProgressFilePath=(new File(tdp ,TargetDirectory.ATTACH_NOTIFICATION_SYNC_FILENAME)).getPath();
		File inProgressFile = new java.io.File(inProgressFilePath);
		return inProgressFile;
	}

	/**
	 * @param myVmId ID of the VM
	 * @return File object to the target's sync file
	 */
	public static File getNotificationSyncFile(String myVmId) {
		String tdp = TargetDirectory.getTargetDirectoryPath(myVmId);
		if (null == tdp) {
			return null;
		}
		File inProgressFile=new File(tdp,ATTACH_NOTIFICATION_SYNC_FILENAME);
		return inProgressFile;
	}

	/**
	 * convenience method for getNotificationSyncFile, using the local value of the VM ID
	 * @return File object to the target's sync file or null if the common directory is missing
	 */
	static String getSyncFilePath() {
		String tdp = TargetDirectory.getTargetDirectoryPath(AttachHandler.getVmId());
		if (null == tdp) {
			return null;
		}
		String syncFilePath=(new File(tdp ,TargetDirectory.ATTACH_NOTIFICATION_SYNC_FILENAME)).getPath();
		return syncFilePath;
	}

}
